/* $Id: InsertXmpServlet.java,v 1.3 2014/10/06 14:13:19 rjs Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * mini sample for inserting XMP metadata in PDF documents
 */

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;

import com.pdflib.PLOPException;
import com.pdflib.plop;

public class InsertXmpServlet extends HttpServlet {
    private static final long serialVersionUID = 5579529181443275023L;

    public void service(HttpServletRequest request, HttpServletResponse response) 
            throws ServletException {
        plop plop = null;

        try {
            /*
             * Set up the search path so the document generated by the encrypt
             * sample can be found.
             */
            final ServletContext context = getServletContext();
            final String searchpath = context.getRealPath("/WEB-INF/data");

            /*
             * Catch problems that application data cannot be accessed.
             */
            if (searchpath == null) {
                throw new ServletException(
                        "Error: Unable to access application data (WAR file not unpacked?)");
            }

            /* input document */
            final String in_filename = "PLOP-datasheet.pdf";

            /* output document */
            final String out_filename = "";
            
            /*
             * Option list for the generated output document. Some alternatives
             * for the option list using the supplied XMP samples:
             * 
             * Insert custom metadata for the GWG (Ghent Workgroup) Ad Ticket
             * scheme (see http://www.gwg.org/Jobtickets.phtml):
             * "metadata={filename=gwg_ad_ticket.xmp validate=xmp2004}"
             * 
             * Insert custom metadata based on Acme's enterprise schema:
             * "metadata={filename=acme.xmp}"
             * 
             * Insert a custom "engineering" schema for scanned documents: This
             * works best with a PDF/A-1 input document (e.g. mini_pdfa1.pdf),
             * and requires XMP input which conforms to PDF/A-1:
             * "metadata={filename=engineering.xmp}"
             * 
             * Insert a PDF/A extension schema after validating the XMP for
             * PDF/A-1. This works best with a PDF/A-1 input document (e.g.
             * mini_pdfa1.pdf), and requires XMP input which conforms to
             * PDF/A-1: "metadata={filename=machine_pdfa1.xmp validate=pdfa1}"
             */
            final String out_optlist = "metadata={filename=gwg_ad_ticket.xmp}";

            plop = new plop();
            
            plop.set_option("searchpath={" + searchpath + "}");

            /* open input file */
            int doc;
            if ((doc = plop.open_document(in_filename, "")) == -1) {
                throw new Exception("Error: " + plop.get_errmsg());
            }

            /* create the output file and add XMP metadata */
            if (plop.create_document(out_filename, out_optlist) == -1) {
                throw new Exception("Error: " + plop.get_errmsg());
            }
            
            final byte[] contents = plop.get_buffer();

            response.setContentType("application/pdf");
            response.setContentLength(contents.length);

            final ServletOutputStream out = response.getOutputStream();
            out.write(contents);
            out.close();

            /* close input and output files */
            plop.close_document(doc, "");
        }
        catch (PLOPException e) {
            throw new ServletException("PLOP exception occurred in "
                    + getClass().getName(), e);
        }
        catch (Exception e) {
            throw new ServletException("Exception occurred in "
                    + getClass().getName(), e);
        }
        finally {
            /* delete the PLOP object */
            if (plop != null)
                plop.delete();
        }
    }
}
